@@ -6,8 +6,9 @@
 #include "serd_internal.h"
 #include "string_utils.h"
 
-#include "serd/serd.h"
+#include <serd/serd.h>
 
+#include <assert.h>
 #include <stdbool.h>
 #include <stdint.h>
 #include <stdlib.h>
@@ -90,7 +91,7 @@ decode_chunk(const uint8_t in[4], uint8_t out[3])
   out[0] = (uint8_t)(((unmap(in[0]) << 2)) | unmap(in[1]) >> 4);
   out[1] = (uint8_t)(((unmap(in[1]) << 4) & 0xF0) | unmap(in[2]) >> 2);
   out[2] = (uint8_t)(((unmap(in[2]) << 6) & 0xC0) | unmap(in[3]));
-  return 1U + (in[2] != '=') + ((in[2] != '=') && (in[3] != '='));
+  return (size_t)1U + (in[2] != '=') + ((in[2] != '=') && (in[3] != '='));
 }
 
 void*
@@ -98,6 +99,9 @@ serd_base64_decode(const uint8_t* const str,
                    const size_t         len,
                    size_t* const        size)
 {
+  assert(str);
+  assert(size);
+
   void* buf = malloc((len * 3) / 4 + 2);
 
   *size = 0;
