@@ -8,12 +8,12 @@
 #include "byte_source.h"
 #include "stack.h"
 
-#include "serd/serd.h"
+#include <serd/serd.h>
 
 #include <assert.h>
 #include <stdbool.h>
+#include <stddef.h>
 #include <stdint.h>
-#include <stdio.h>
 
 #ifdef SERD_STACK_CHECK
 #  define SERD_STACK_ASSERT_TOP(reader, ref) \
@@ -110,15 +110,15 @@ SerdStatus
 read_turtleTrigDoc(SerdReader* reader);
 
 static inline int
-peek_byte(SerdReader* reader)
+peek_byte(SerdReader* const reader)
 {
   SerdByteSource* source = &reader->source;
 
-  return source->eof ? EOF : (int)source->read_buf[source->read_head];
+  return source->eof ? -1 : (int)source->read_buf[source->read_head];
 }
 
 static inline SerdStatus
-skip_byte(SerdReader* reader, const int byte)
+skip_byte(SerdReader* const reader, const int byte)
 {
   (void)byte;
 
@@ -128,7 +128,7 @@ skip_byte(SerdReader* reader, const int byte)
 }
 
 static inline int SERD_NODISCARD
-eat_byte_safe(SerdReader* reader, const int byte)
+eat_byte_safe(SerdReader* const reader, const int byte)
 {
   (void)byte;
 
@@ -139,7 +139,7 @@ eat_byte_safe(SerdReader* reader, const int byte)
 }
 
 static inline int SERD_NODISCARD
-eat_byte_check(SerdReader* reader, const int byte)
+eat_byte_check(SerdReader* const reader, const int byte)
 {
   const int c = peek_byte(reader);
   if (c != byte) {
@@ -150,7 +150,7 @@ eat_byte_check(SerdReader* reader, const int byte)
 }
 
 static inline SerdStatus
-eat_string(SerdReader* reader, const char* str, unsigned n)
+eat_string(SerdReader* const reader, const char* const str, const unsigned n)
 {
   for (unsigned i = 0; i < n; ++i) {
     if (!eat_byte_check(reader, ((const uint8_t*)str)[i])) {
@@ -161,9 +161,9 @@ eat_string(SerdReader* reader, const char* str, unsigned n)
 }
 
 static inline SerdStatus
-push_byte(SerdReader* reader, Ref ref, const int c)
+push_byte(SerdReader* const reader, const Ref ref, const int c)
 {
-  assert(c != EOF);
+  assert(c >= 0);
   SERD_STACK_ASSERT_TOP(reader, ref);
 
   uint8_t* const  s    = (uint8_t*)serd_stack_push(&reader->stack, 1);
@@ -180,7 +180,10 @@ push_byte(SerdReader* reader, Ref ref, const int c)
 }
 
 static inline void
-push_bytes(SerdReader* reader, Ref ref, const uint8_t* bytes, unsigned len)
+push_bytes(SerdReader* const    reader,
+           const Ref            ref,
+           const uint8_t* const bytes,
+           const unsigned       len)
 {
   for (unsigned i = 0; i < len; ++i) {
     push_byte(reader, ref, bytes[i]);
