@@ -4,7 +4,7 @@
 #ifndef SERD_SRC_STRING_UTILS_H
 #define SERD_SRC_STRING_UTILS_H
 
-#include "serd/serd.h"
+#include <serd/serd.h>
 
 #include <stdbool.h>
 #include <stddef.h>
@@ -51,17 +51,7 @@ is_xdigit(const int c)
 static inline bool
 is_space(const char c)
 {
-  switch (c) {
-  case ' ':
-  case '\f':
-  case '\n':
-  case '\r':
-  case '\t':
-  case '\v':
-    return true;
-  default:
-    return false;
-  }
+  return c == ' ' || (c >= '\t' && c <= '\r');
 }
 
 static inline bool
@@ -101,19 +91,23 @@ serd_to_upper(const char c)
   return (char)((c >= 'a' && c <= 'z') ? c - 32 : c);
 }
 
-static inline int
-serd_strncasecmp(const char* s1, const char* s2, size_t n)
+SERD_PURE_FUNC static inline int
+serd_strcasecmp(const char* s1, const char* s2)
 {
-  for (; n > 0 && *s2; s1++, s2++, --n) {
-    if (serd_to_upper(*s1) != serd_to_upper(*s2)) {
-      return ((*(const uint8_t*)s1 < *(const uint8_t*)s2) ? -1 : +1);
+  while (*s1 && *s2) {
+    const char c1 = serd_to_upper(*s1++);
+    const char c2 = serd_to_upper(*s2++);
+    if (c1 != c2) {
+      return (c1 < c2) ? -1 : +1;
     }
   }
 
-  return 0;
+  const char c1 = serd_to_upper(*s1);
+  const char c2 = serd_to_upper(*s2);
+  return (c1 == c2) ? 0 : (c1 < c2) ? -1 : +1;
 }
 
-static inline uint32_t
+static inline uint8_t
 utf8_num_bytes(const uint8_t leading)
 {
   return ((leading & 0x80U) == 0x00U)   ? 1U  // Starts with `0'
@@ -125,18 +119,18 @@ utf8_num_bytes(const uint8_t leading)
 
 /// Return the code point of a UTF-8 character with known length
 static inline uint32_t
-parse_counted_utf8_char(const uint8_t* utf8, size_t size)
+parse_counted_utf8_char(const uint8_t* const utf8, const uint8_t size)
 {
   uint32_t c = utf8[0] & ((1U << (8U - size)) - 1U);
   for (size_t i = 1; i < size; ++i) {
-    c = (c << 6) | (utf8[i] & 0x3FU);
+    c = (c << 6U) | (utf8[i] & 0x3FU);
   }
   return c;
 }
 
 /// Parse a UTF-8 character, set *size to the length, and return the code point
 static inline uint32_t
-parse_utf8_char(const uint8_t* utf8, size_t* size)
+parse_utf8_char(const uint8_t* const utf8, uint8_t* const size)
 {
   switch (*size = utf8_num_bytes(utf8[0])) {
   case 1:
