@@ -5,8 +5,9 @@
 #include "uri_utils.h"
 #include "warnings.h"
 
-#include "serd/serd.h"
+#include <serd/serd.h>
 
+#include <assert.h>
 #include <stdbool.h>
 #include <stdint.h>
 #include <stdio.h>
@@ -14,8 +15,10 @@
 #include <string.h>
 
 const uint8_t*
-serd_uri_to_path(const uint8_t* uri)
+serd_uri_to_path(const uint8_t* const uri)
 {
+  assert(uri);
+
   const uint8_t* path = uri;
   if (!is_windows_path(uri) && serd_uri_string_has_scheme(uri)) {
     if (!!strncmp((const char*)uri, "file:", 5)) {
@@ -42,6 +45,8 @@ serd_uri_to_path(const uint8_t* uri)
 uint8_t*
 serd_file_uri_parse(const uint8_t* const uri, uint8_t** const hostname)
 {
+  assert(uri);
+
   const uint8_t* path = uri;
   if (hostname) {
     *hostname = NULL;
@@ -113,6 +118,9 @@ serd_uri_string_has_scheme(const uint8_t* utf8)
 SerdStatus
 serd_uri_parse(const uint8_t* const utf8, SerdURI* const out)
 {
+  assert(utf8);
+  assert(out);
+
   *out = SERD_URI_NULL;
 
   const uint8_t* ptr = utf8;
@@ -293,6 +301,10 @@ serd_uri_resolve(const SerdURI* const r,
                  const SerdURI* const base,
                  SerdURI* const       t)
 {
+  assert(r);
+  assert(base);
+  assert(t);
+
   if (!base->scheme.len) {
     *t = *r; // Don't resolve against non-absolute URIs
     return;
@@ -332,7 +344,7 @@ serd_uri_resolve(const SerdURI* const r,
 
 /** Write the path of `uri` starting at index `i` */
 static size_t
-write_path_tail(SerdSink             sink,
+write_path_tail(const SerdSink       sink,
                 void* const          stream,
                 const SerdURI* const uri,
                 const size_t         i)
@@ -360,7 +372,7 @@ write_path_tail(SerdSink             sink,
 
 /** Write the path of `uri` relative to the path of `base`. */
 static size_t
-write_rel_path(SerdSink             sink,
+write_rel_path(const SerdSink       sink,
                void* const          stream,
                const SerdURI* const uri,
                const SerdURI* const base)
@@ -401,7 +413,7 @@ write_rel_path(SerdSink             sink,
 }
 
 static uint8_t
-serd_uri_path_starts_without_slash(const SerdURI* uri)
+serd_uri_path_starts_without_slash(const SerdURI* const uri)
 {
   return ((uri->path_base.len || uri->path.len) &&
           ((!uri->path_base.len || uri->path_base.buf[0] != '/') &&
@@ -413,9 +425,12 @@ size_t
 serd_uri_serialise_relative(const SerdURI* const uri,
                             const SerdURI* const base,
                             const SerdURI* const root,
-                            SerdSink             sink,
+                            const SerdSink       sink,
                             void* const          stream)
 {
+  assert(uri);
+  assert(sink);
+
   size_t     len = 0;
   const bool relative =
     root ? uri_is_under(uri, root) : uri_is_related(uri, base);
@@ -426,7 +441,7 @@ serd_uri_serialise_relative(const SerdURI* const uri,
 
   SERD_DISABLE_NULL_WARNINGS
 
-  if (!relative || (!len && base->query.buf)) {
+  if (!relative || (!len && base && base->query.buf)) {
     if (uri->scheme.buf) {
       len += sink(uri->scheme.buf, uri->scheme.len, stream);
       len += sink(":", 1, stream);
@@ -466,7 +481,11 @@ serd_uri_serialise_relative(const SerdURI* const uri,
 
 /// See http://tools.ietf.org/html/rfc3986#section-5.3
 size_t
-serd_uri_serialise(const SerdURI* const uri, SerdSink sink, void* const stream)
+serd_uri_serialise(const SerdURI* const uri,
+                   const SerdSink       sink,
+                   void* const          stream)
 {
+  assert(uri);
+  assert(sink);
   return serd_uri_serialise_relative(uri, NULL, NULL, sink, stream);
 }
