@@ -10,8 +10,9 @@
 #include "uri_utils.h"
 #include "warnings.h"
 
-#include "serd/serd.h"
+#include <serd/serd.h>
 
+#include <assert.h>
 #include <errno.h>
 #include <stdarg.h>
 #include <stdbool.h>
@@ -51,9 +52,8 @@ static const WriteContext WRITE_CONTEXT_NULL = {CTX_NAMED,
                                                 0U};
 
 typedef enum {
-  SEP_NONE,        ///< Sentinel before the start of a document
-  SEP_NODE,        ///< Sentinel after a node
-  SEP_NEWLINE,     ///< Sentinel after a node
+  SEP_NONE,        ///< Sentinel after "nothing"
+  SEP_NEWLINE,     ///< Sentinel after a line end
   SEP_END_DIRECT,  ///< End of a directive (like "@prefix")
   SEP_END_S,       ///< End of a subject ('.')
   SEP_END_P,       ///< End of a predicate (';')
@@ -88,7 +88,6 @@ typedef struct {
 #define NIL '\0'
 
 static const SepRule rules[] = {
-  {NIL, +0, SEP_NONE, SEP_NONE, SEP_NONE},
   {NIL, +0, SEP_NONE, SEP_NONE, SEP_NONE},
   {'\n', 0, SEP_NONE, SEP_NONE, SEP_NONE},
   {'.', +0, SEP_EACH, SEP_NONE, SEP_EACH},
@@ -144,13 +143,7 @@ write_node(SerdWriter*        writer,
            SerdStatementFlags flags);
 
 SERD_NODISCARD static bool
-supports_abbrev(const SerdWriter* writer)
-{
-  return writer->syntax == SERD_TURTLE || writer->syntax == SERD_TRIG;
-}
-
-SERD_NODISCARD static bool
-supports_uriref(const SerdWriter* writer)
+supports_abbrev(const SerdWriter* const writer)
 {
   return writer->syntax == SERD_TURTLE || writer->syntax == SERD_TRIG;
 }
@@ -169,7 +162,7 @@ free_context(WriteContext* const ctx)
 
 SERD_LOG_FUNC(3, 4)
 static SerdStatus
-w_err(SerdWriter* writer, SerdStatus st, const char* fmt, ...)
+w_err(SerdWriter* const writer, const SerdStatus st, const char* const fmt, ...)
 {
   /* TODO: This results in errors with no file information, which is not
      helpful when re-serializing a file (particularly for "undefined
@@ -186,7 +179,7 @@ w_err(SerdWriter* writer, SerdStatus st, const char* fmt, ...)
 }
 
 static void
-copy_node(SerdNode* dst, const SerdNode* src)
+copy_node(SerdNode* const dst, const SerdNode* const src)
 {
   const size_t   new_size = src->n_bytes + 1U;
   uint8_t* const new_buf  = (uint8_t*)realloc((char*)dst->buf, new_size);
@@ -217,7 +210,7 @@ push_context(SerdWriter* const writer,
 }
 
 static void
-pop_context(SerdWriter* writer)
+pop_context(SerdWriter* const writer)
 {
   // Replace the current context with the top of the stack
   free_context(&writer->context);
@@ -230,7 +223,7 @@ pop_context(SerdWriter* writer)
 }
 
 SERD_NODISCARD static size_t
-sink(const void* buf, size_t len, SerdWriter* writer)
+sink(const void* const buf, const size_t len, SerdWriter* const writer)
 {
   const size_t written = serd_byte_sink_write(buf, len, &writer->byte_sink);
   if (written != len) {
@@ -246,7 +239,7 @@ sink(const void* buf, size_t len, SerdWriter* writer)
 }
 
 SERD_NODISCARD static inline SerdStatus
-esink(const void* buf, size_t len, SerdWriter* writer)
+esink(const void* const buf, const size_t len, SerdWriter* const writer)
 {
   return sink(buf, len, writer) == len ? SERD_SUCCESS : SERD_ERR_BAD_WRITE;
 }
@@ -254,10 +247,10 @@ esink(const void* buf, size_t len, SerdWriter* writer)
 // Write a single character, as an escape for single byte characters
 // (Caller prints any single byte characters that don't need escaping)
 static size_t
-write_character(SerdWriter*    writer,
-                const uint8_t* utf8,
-                size_t*        size,
-                SerdStatus*    st)
+write_character(SerdWriter* const    writer,
+                const uint8_t* const utf8,
+                uint8_t* const       size,
+                SerdStatus* const    st)
 {
   char           escape[11] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
   const uint32_t c          = parse_utf8_char(utf8, size);
@@ -290,28 +283,15 @@ write_character(SerdWriter*    writer,
 SERD_NODISCARD static bool
 uri_must_escape(const uint8_t c)
 {
-  switch (c) {
-  case ' ':
-  case '"':
-  case '<':
-  case '>':
-  case '\\':
-  case '^':
-  case '`':
-  case '{':
-  case '|':
-  case '}':
-    return true;
-  default:
-    return !in_range(c, 0x20, 0x7E);
-  }
+  return (c == '"') || (c == '<') || (c == '>') || (c == '\\') || (c == '^') ||
+         (c == '`') || in_range(c, '{', '}') || !in_range(c, 0x21, 0x7E);
 }
 
 static size_t
-write_uri(SerdWriter*    writer,
-          const uint8_t* utf8,
-          size_t         n_bytes,
-          SerdStatus*    st)
+write_uri(SerdWriter* const    writer,
+          const uint8_t* const utf8,
+          const size_t         n_bytes,
+          SerdStatus* const    st)
 {
   size_t len = 0;
   for (size_t i = 0; i < n_bytes;) {
@@ -335,14 +315,14 @@ write_uri(SerdWriter*    writer,
     }
 
     // Write UTF-8 character
-    size_t size = 0;
+    uint8_t size = 0U;
     len += write_character(writer, utf8 + i, &size, st);
     i += size;
     if (*st && (writer->style & SERD_STYLE_STRICT)) {
       break;
     }
 
-    if (size == 0) {
+    if (!size) {
       // Corrupt input, write percent-encoded bytes and scan to next start
       char escape[4] = {0, 0, 0, 0};
       for (; i < n_bytes && (utf8[i] & 0x80); ++i) {
@@ -356,7 +336,9 @@ write_uri(SerdWriter*    writer,
 }
 
 SERD_NODISCARD static SerdStatus
-ewrite_uri(SerdWriter* writer, const uint8_t* utf8, size_t n_bytes)
+ewrite_uri(SerdWriter* const    writer,
+           const uint8_t* const utf8,
+           const size_t         n_bytes)
 {
   SerdStatus st = SERD_SUCCESS;
   write_uri(writer, utf8, n_bytes, &st);
@@ -367,7 +349,7 @@ ewrite_uri(SerdWriter* writer, const uint8_t* utf8, size_t n_bytes)
 }
 
 SERD_NODISCARD static SerdStatus
-write_uri_from_node(SerdWriter* writer, const SerdNode* node)
+write_uri_from_node(SerdWriter* const writer, const SerdNode* const node)
 {
   return ewrite_uri(writer, node->buf, node->n_bytes);
 }
@@ -375,42 +357,23 @@ write_uri_from_node(SerdWriter* writer, const SerdNode* node)
 static bool
 lname_must_escape(const uint8_t c)
 {
-  /* This arbitrary list of characters, most of which have nothing to do with
-     Turtle, must be handled as special cases here because the RDF and SPARQL
-     WGs are apparently intent on making the once elegant Turtle a baroque
-     and inconsistent mess, throwing elegance and extensibility completely
-     out the window for no good reason.
+  /* Most of these characters have nothing to do with Turtle, but were taken
+     from SPARQL and mashed into the Turtle grammar (despite not being used)
+     with RDF 1.1.  So now Turtle is a mess because the SPARQL grammar is
+     poorly designed and didn't use a leading character to distinguish things
+     like path patterns like it should have.
 
-     Note '-', '.', and '_' are also in PN_LOCAL_ESC, but are valid unescaped
-     in local names, so they are not escaped here. */
+     Note that '-', '.', and '_' are also in PN_LOCAL_ESC, but are valid
+     unescaped in local names, so they are not escaped here. */
 
-  switch (c) {
-  case '\'':
-  case '!':
-  case '#':
-  case '$':
-  case '%':
-  case '&':
-  case '(':
-  case ')':
-  case '*':
-  case '+':
-  case ',':
-  case '/':
-  case ';':
-  case '=':
-  case '?':
-  case '@':
-  case '~':
-    return true;
-  default:
-    break;
-  }
-  return false;
+  return (c == '!') || (c == '/') || (c == ';') || (c == '=') || (c == '?') ||
+         (c == '@') || (c == '~') || in_range(c, '#', ',');
 }
 
 SERD_NODISCARD static SerdStatus
-write_lname(SerdWriter* writer, const uint8_t* utf8, size_t n_bytes)
+write_lname(SerdWriter* const    writer,
+            const uint8_t* const utf8,
+            const size_t         n_bytes)
 {
   SerdStatus st = SERD_SUCCESS;
   for (size_t i = 0; i < n_bytes; ++i) {
@@ -436,10 +399,10 @@ write_lname(SerdWriter* writer, const uint8_t* utf8, size_t n_bytes)
 }
 
 SERD_NODISCARD static SerdStatus
-write_text(SerdWriter*    writer,
-           TextContext    ctx,
-           const uint8_t* utf8,
-           size_t         n_bytes)
+write_text(SerdWriter* const    writer,
+           const TextContext    ctx,
+           const uint8_t* const utf8,
+           const size_t         n_bytes)
 {
   size_t     n_consecutive_quotes = 0;
   SerdStatus st                   = SERD_SUCCESS;
@@ -525,19 +488,19 @@ write_text(SerdWriter*    writer,
     }
 
     // Write UTF-8 character
-    size_t size = 0;
+    uint8_t size = 0U;
     write_character(writer, utf8 + i - 1, &size, &st);
     if (st && (writer->style & SERD_STYLE_STRICT)) {
       return st;
     }
 
-    if (size == 0) {
+    if (size > 0U) {
+      i += size - 1U;
+    } else {
       // Corrupt input, write replacement character and scan to the next start
       st = esink(replacement_char, sizeof(replacement_char), writer);
-      for (; i < n_bytes && (utf8[i] & 0x80); ++i) {
+      for (; i < n_bytes && (utf8[i] & 0x80U); ++i) {
       }
-    } else {
-      i += size - 1;
     }
   }
 
@@ -550,7 +513,7 @@ typedef struct {
 } UriSinkContext;
 
 SERD_NODISCARD static size_t
-uri_sink(const void* buf, size_t len, void* stream)
+uri_sink(const void* const buf, const size_t len, void* const stream)
 {
   UriSinkContext* const context = (UriSinkContext*)stream;
   SerdWriter* const     writer  = context->writer;
@@ -559,7 +522,7 @@ uri_sink(const void* buf, size_t len, void* stream)
 }
 
 SERD_NODISCARD static SerdStatus
-write_newline(SerdWriter* writer)
+write_newline(SerdWriter* const writer)
 {
   SerdStatus st = SERD_SUCCESS;
 
@@ -572,7 +535,7 @@ write_newline(SerdWriter* writer)
 }
 
 SERD_NODISCARD static SerdStatus
-write_sep(SerdWriter* writer, const Sep sep)
+write_sep(SerdWriter* const writer, const Sep sep)
 {
   SerdStatus           st   = SERD_SUCCESS;
   const SepRule* const rule = &rules[sep];
@@ -626,7 +589,7 @@ write_sep(SerdWriter* writer, const Sep sep)
 }
 
 static void
-free_anon_stack(SerdWriter* writer)
+free_anon_stack(SerdWriter* const writer)
 {
   while (!serd_stack_is_empty(&writer->anon_stack)) {
     pop_context(writer);
@@ -634,7 +597,7 @@ free_anon_stack(SerdWriter* writer)
 }
 
 static SerdStatus
-reset_context(SerdWriter* writer, const unsigned flags)
+reset_context(SerdWriter* const writer, const unsigned flags)
 {
   free_anon_stack(writer);
 
@@ -654,31 +617,45 @@ reset_context(SerdWriter* writer, const unsigned flags)
   return SERD_SUCCESS;
 }
 
+// Return the name of the XSD datatype referred to by `datatype`, if any
+static const char*
+get_xsd_name(const SerdEnv* const env, const SerdNode* const datatype)
+{
+  const char* const datatype_str = (const char*)datatype->buf;
+
+  if (datatype->type == SERD_URI &&
+      (!strncmp(datatype_str, NS_XSD, sizeof(NS_XSD) - 1))) {
+    return datatype_str + sizeof(NS_XSD) - 1U;
+  }
+
+  if (datatype->type == SERD_CURIE) {
+    SerdChunk prefix = {NULL, 0};
+    SerdChunk suffix = {NULL, 0};
+    // We can be a bit lazy/presumptive here due to grammar limitations
+    if (!serd_env_expand(env, datatype, &prefix, &suffix)) {
+      if (!strcmp((const char*)prefix.buf, NS_XSD)) {
+        return (const char*)suffix.buf;
+      }
+    }
+  }
+
+  return "";
+}
+
 SERD_NODISCARD static SerdStatus
-write_literal(SerdWriter*        writer,
-              const SerdNode*    node,
-              const SerdNode*    datatype,
-              const SerdNode*    lang,
-              SerdStatementFlags flags)
+write_literal(SerdWriter* const        writer,
+              const SerdNode* const    node,
+              const SerdNode* const    datatype,
+              const SerdNode* const    lang,
+              const SerdStatementFlags flags)
 {
   SerdStatus st = SERD_SUCCESS;
 
   if (supports_abbrev(writer) && datatype && datatype->buf) {
-    const char* type_uri = (const char*)datatype->buf;
-    if (!strncmp(type_uri, NS_XSD, sizeof(NS_XSD) - 1) &&
-        (!strcmp(type_uri + sizeof(NS_XSD) - 1, "boolean") ||
-         !strcmp(type_uri + sizeof(NS_XSD) - 1, "integer"))) {
-      return esink(node->buf, node->n_bytes, writer);
-    }
-
-    if (!strncmp(type_uri, NS_XSD, sizeof(NS_XSD) - 1) &&
-        !strcmp(type_uri + sizeof(NS_XSD) - 1, "decimal") &&
-        strchr((const char*)node->buf, '.') &&
-        node->buf[node->n_bytes - 1] != '.') {
-      /* xsd:decimal literals without trailing digits, e.g. "5.", can
-         not be written bare in Turtle.  We could add a 0 which is
-         prettier, but changes the text and breaks round tripping.
-      */
+    const char* const xsd_name = get_xsd_name(writer->env, datatype);
+    if (!strcmp(xsd_name, "boolean") || !strcmp(xsd_name, "integer") ||
+        (!strcmp(xsd_name, "decimal") && strchr((const char*)node->buf, '.') &&
+         node->buf[node->n_bytes - 1] != '.')) {
       return esink(node->buf, node->n_bytes, writer);
     }
   }
@@ -687,18 +664,18 @@ write_literal(SerdWriter*        writer,
       (node->flags & (SERD_HAS_NEWLINE | SERD_HAS_QUOTE))) {
     TRY(st, esink("\"\"\"", 3, writer));
     TRY(st, write_text(writer, WRITE_LONG_STRING, node->buf, node->n_bytes));
-    TRY(st, esink("\"\"\"", 3, writer));
+    st = esink("\"\"\"", 3, writer);
   } else {
     TRY(st, esink("\"", 1, writer));
     TRY(st, write_text(writer, WRITE_STRING, node->buf, node->n_bytes));
-    TRY(st, esink("\"", 1, writer));
+    st = esink("\"", 1, writer);
   }
   if (lang && lang->buf) {
     TRY(st, esink("@", 1, writer));
-    TRY(st, esink(lang->buf, lang->n_bytes, writer));
+    st = esink(lang->buf, lang->n_bytes, writer);
   } else if (datatype && datatype->buf) {
     TRY(st, esink("^^", 2, writer));
-    return write_node(writer, datatype, NULL, NULL, FIELD_NONE, flags);
+    st = write_node(writer, datatype, NULL, NULL, FIELD_NONE, flags);
   }
 
   return st;
@@ -706,7 +683,7 @@ write_literal(SerdWriter*        writer,
 
 // Return true iff `buf` is a valid prefixed name prefix or suffix
 static bool
-is_name(const uint8_t* buf, const size_t len)
+is_name(const uint8_t* const buf, const size_t len)
 {
   // TODO: This is more strict than it should be
   for (size_t i = 0; i < len; ++i) {
@@ -719,9 +696,9 @@ is_name(const uint8_t* buf, const size_t len)
 }
 
 SERD_NODISCARD static SerdStatus
-write_uri_node(SerdWriter* const writer,
-               const SerdNode*   node,
-               const Field       field)
+write_uri_node(SerdWriter* const     writer,
+               const SerdNode* const node,
+               const Field           field)
 {
   SerdStatus st     = SERD_SUCCESS;
   SerdNode   prefix = SERD_NODE_NULL;
@@ -748,7 +725,8 @@ write_uri_node(SerdWriter* const writer,
     }
   }
 
-  if (!has_scheme && !supports_uriref(writer) &&
+  if (!has_scheme &&
+      (writer->syntax == SERD_NTRIPLES || writer->syntax == SERD_NQUADS) &&
       !serd_env_get_base_uri(writer->env, NULL)->buf) {
     return w_err(writer,
                  SERD_ERR_BAD_ARG,
@@ -767,8 +745,8 @@ write_uri_node(SerdWriter* const writer,
     serd_uri_parse(node->buf, &uri);
     SERD_RESTORE_WARNINGS
     serd_uri_resolve(&uri, &in_base_uri, &abs_uri);
-    bool           rooted = uri_is_under(&writer->base_uri, &writer->root_uri);
-    SerdURI*       root   = rooted ? &writer->root_uri : &writer->base_uri;
+    const bool     rooted = uri_is_under(&writer->base_uri, &writer->root_uri);
+    const SerdURI* root   = rooted ? &writer->root_uri : &writer->base_uri;
     UriSinkContext ctx    = {writer, SERD_SUCCESS};
     if (!uri_is_under(&abs_uri, root) || writer->syntax == SERD_NTRIPLES ||
         writer->syntax == SERD_NQUADS) {
@@ -815,7 +793,7 @@ write_curie(SerdWriter* const writer, const SerdNode* const node)
 
 SERD_NODISCARD static SerdStatus
 write_blank(SerdWriter* const        writer,
-            const SerdNode*          node,
+            const SerdNode* const    node,
             const Field              field,
             const SerdStatementFlags flags)
 {
@@ -854,47 +832,31 @@ write_blank(SerdWriter* const        writer,
 }
 
 SERD_NODISCARD static SerdStatus
-write_node(SerdWriter*        writer,
-           const SerdNode*    node,
-           const SerdNode*    datatype,
-           const SerdNode*    lang,
-           Field              field,
-           SerdStatementFlags flags)
+write_node(SerdWriter* const        writer,
+           const SerdNode* const    node,
+           const SerdNode* const    datatype,
+           const SerdNode* const    lang,
+           const Field              field,
+           const SerdStatementFlags flags)
 {
-  SerdStatus st = SERD_SUCCESS;
-
-  switch (node->type) {
-  case SERD_NOTHING:
-    break;
-  case SERD_LITERAL:
-    st = write_literal(writer, node, datatype, lang, flags);
-    break;
-  case SERD_URI:
-    st = write_uri_node(writer, node, field);
-    break;
-  case SERD_CURIE:
-    st = write_curie(writer, node);
-    break;
-  case SERD_BLANK:
-    st = write_blank(writer, node, field, flags);
-    break;
-  }
-
-  if (node->type != SERD_BLANK) {
-    writer->last_sep = SEP_NODE;
-  }
-
-  return st;
+  return (node->type == SERD_LITERAL)
+           ? write_literal(writer, node, datatype, lang, flags)
+         : (node->type == SERD_URI)   ? write_uri_node(writer, node, field)
+         : (node->type == SERD_CURIE) ? write_curie(writer, node)
+         : (node->type == SERD_BLANK) ? write_blank(writer, node, field, flags)
+                                      : SERD_SUCCESS;
 }
 
 static bool
-is_resource(const SerdNode* node)
+is_resource(const SerdNode* const node)
 {
-  return node && node->buf && node->type > SERD_LITERAL;
+  return node->buf && node->type > SERD_LITERAL;
 }
 
 SERD_NODISCARD static SerdStatus
-write_pred(SerdWriter* writer, SerdStatementFlags flags, const SerdNode* pred)
+write_pred(SerdWriter* const        writer,
+           const SerdStatementFlags flags,
+           const SerdNode* const    pred)
 {
   SerdStatus st = SERD_SUCCESS;
 
@@ -908,12 +870,12 @@ write_pred(SerdWriter* writer, SerdStatementFlags flags, const SerdNode* pred)
 }
 
 SERD_NODISCARD static SerdStatus
-write_list_next(SerdWriter*        writer,
-                SerdStatementFlags flags,
-                const SerdNode*    predicate,
-                const SerdNode*    object,
-                const SerdNode*    datatype,
-                const SerdNode*    lang)
+write_list_next(SerdWriter* const        writer,
+                const SerdStatementFlags flags,
+                const SerdNode* const    predicate,
+                const SerdNode* const    object,
+                const SerdNode* const    datatype,
+                const SerdNode* const    lang)
 {
   SerdStatus st = SERD_SUCCESS;
 
@@ -932,7 +894,7 @@ write_list_next(SerdWriter*        writer,
 }
 
 SERD_NODISCARD static SerdStatus
-terminate_context(SerdWriter* writer)
+terminate_context(SerdWriter* const writer)
 {
   SerdStatus st = SERD_SUCCESS;
 
@@ -948,19 +910,23 @@ terminate_context(SerdWriter* writer)
 }
 
 SerdStatus
-serd_writer_write_statement(SerdWriter*        writer,
-                            SerdStatementFlags flags,
-                            const SerdNode*    graph,
-                            const SerdNode*    subject,
-                            const SerdNode*    predicate,
-                            const SerdNode*    object,
-                            const SerdNode*    datatype,
-                            const SerdNode*    lang)
+serd_writer_write_statement(SerdWriter* const     writer,
+                            SerdStatementFlags    flags,
+                            const SerdNode* const graph,
+                            const SerdNode* const subject,
+                            const SerdNode* const predicate,
+                            const SerdNode* const object,
+                            const SerdNode* const datatype,
+                            const SerdNode* const lang)
 {
+  assert(writer);
+  assert(subject);
+  assert(predicate);
+  assert(object);
+
   SerdStatus st = SERD_SUCCESS;
 
-  if (!is_resource(subject) || !is_resource(predicate) || !object ||
-      !object->buf) {
+  if (!is_resource(subject) || !is_resource(predicate) || !object->buf) {
     return SERD_ERR_BAD_ARG;
   }
 
@@ -994,8 +960,8 @@ serd_writer_write_statement(SerdWriter*        writer,
       (!graph && writer->context.graph.type)) {
     TRY(st, terminate_context(writer));
     reset_context(writer, RESET_GRAPH | RESET_INDENT);
+    TRY(st, write_newline(writer));
     if (graph) {
-      TRY(st, write_newline(writer));
       TRY(st, write_node(writer, graph, datatype, lang, FIELD_GRAPH, flags));
       TRY(st, write_sep(writer, SEP_GRAPH_BEGIN));
       copy_node(&writer->context.graph, graph);
@@ -1051,24 +1017,24 @@ serd_writer_write_statement(SerdWriter*        writer,
 
   } else {
     // No abbreviation
-    if (serd_stack_is_empty(&writer->anon_stack)) {
-      if (writer->context.subject.type) {
-        TRY(st, write_sep(writer, SEP_END_S));
-      }
 
-      if (writer->last_sep == SEP_END_S || writer->last_sep == SEP_END_DIRECT) {
-        TRY(st, write_newline(writer));
-      }
+    if (!serd_stack_is_empty(&writer->anon_stack)) {
+      return SERD_ERR_BAD_ARG;
+    }
 
-      TRY(st, write_node(writer, subject, NULL, NULL, FIELD_SUBJECT, flags));
-      if ((flags & (SERD_ANON_S_BEGIN | SERD_LIST_S_BEGIN))) {
-        TRY(st, write_sep(writer, SEP_ANON_S_P));
-      } else {
-        TRY(st, write_sep(writer, SEP_S_P));
-      }
+    if (writer->context.subject.type) {
+      TRY(st, write_sep(writer, SEP_END_S));
+    }
 
-    } else {
+    if (writer->last_sep == SEP_END_S || writer->last_sep == SEP_END_DIRECT) {
+      TRY(st, write_newline(writer));
+    }
+
+    TRY(st, write_node(writer, subject, NULL, NULL, FIELD_SUBJECT, flags));
+    if ((flags & (SERD_ANON_S_BEGIN | SERD_LIST_S_BEGIN))) {
       TRY(st, write_sep(writer, SEP_ANON_S_P));
+    } else {
+      TRY(st, write_sep(writer, SEP_S_P));
     }
 
     reset_context(writer, 0U);
@@ -1104,8 +1070,10 @@ serd_writer_write_statement(SerdWriter*        writer,
 }
 
 SerdStatus
-serd_writer_end_anon(SerdWriter* writer, const SerdNode* node)
+serd_writer_end_anon(SerdWriter* const writer, const SerdNode* const node)
 {
+  assert(writer);
+
   SerdStatus st = SERD_SUCCESS;
 
   if (writer->syntax == SERD_NTRIPLES || writer->syntax == SERD_NQUADS) {
@@ -1133,8 +1101,10 @@ serd_writer_end_anon(SerdWriter* writer, const SerdNode* node)
 }
 
 SerdStatus
-serd_writer_finish(SerdWriter* writer)
+serd_writer_finish(SerdWriter* const writer)
 {
+  assert(writer);
+
   const SerdStatus st0 = terminate_context(writer);
   const SerdStatus st1 = serd_byte_sink_flush(&writer->byte_sink);
   free_anon_stack(writer);
@@ -1143,13 +1113,16 @@ serd_writer_finish(SerdWriter* writer)
 }
 
 SerdWriter*
-serd_writer_new(SerdSyntax     syntax,
-                SerdStyle      style,
-                SerdEnv*       env,
-                const SerdURI* base_uri,
-                SerdSink       ssink,
-                void*          stream)
+serd_writer_new(const SerdSyntax     syntax,
+                const SerdStyle      style,
+                SerdEnv* const       env,
+                const SerdURI* const base_uri,
+                SerdSink             ssink,
+                void* const          stream)
 {
+  assert(env);
+  assert(ssink);
+
   const WriteContext context = WRITE_CONTEXT_NULL;
   SerdWriter*        writer  = (SerdWriter*)calloc(1, sizeof(SerdWriter));
 
@@ -1168,17 +1141,22 @@ serd_writer_new(SerdSyntax     syntax,
 }
 
 void
-serd_writer_set_error_sink(SerdWriter*   writer,
-                           SerdErrorSink error_sink,
-                           void*         error_handle)
+serd_writer_set_error_sink(SerdWriter* const   writer,
+                           const SerdErrorSink error_sink,
+                           void* const         error_handle)
 {
+  assert(writer);
+  assert(error_sink);
   writer->error_sink   = error_sink;
   writer->error_handle = error_handle;
 }
 
 void
-serd_writer_chop_blank_prefix(SerdWriter* writer, const uint8_t* prefix)
+serd_writer_chop_blank_prefix(SerdWriter* const    writer,
+                              const uint8_t* const prefix)
 {
+  assert(writer);
+
   free(writer->bprefix);
   writer->bprefix_len = 0;
   writer->bprefix     = NULL;
@@ -1192,8 +1170,10 @@ serd_writer_chop_blank_prefix(SerdWriter* writer, const uint8_t* prefix)
 }
 
 SerdStatus
-serd_writer_set_base_uri(SerdWriter* writer, const SerdNode* uri)
+serd_writer_set_base_uri(SerdWriter* const writer, const SerdNode* const uri)
 {
+  assert(writer);
+
   SerdStatus st = SERD_SUCCESS;
 
   TRY(st, serd_env_set_base_uri(writer->env, uri));
@@ -1205,7 +1185,6 @@ serd_writer_set_base_uri(SerdWriter* writer, const SerdNode* uri)
     TRY(st, esink("@base <", 7, writer));
     TRY(st, esink(uri->buf, uri->n_bytes, writer));
     TRY(st, esink(">", 1, writer));
-    writer->last_sep = SEP_NODE;
     TRY(st, write_sep(writer, SEP_END_DIRECT));
   }
 
@@ -1213,8 +1192,10 @@ serd_writer_set_base_uri(SerdWriter* writer, const SerdNode* uri)
 }
 
 SerdStatus
-serd_writer_set_root_uri(SerdWriter* writer, const SerdNode* uri)
+serd_writer_set_root_uri(SerdWriter* const writer, const SerdNode* const uri)
 {
+  assert(writer);
+
   serd_node_free(&writer->root_node);
 
   if (uri && uri->buf) {
@@ -1231,10 +1212,14 @@ serd_writer_set_root_uri(SerdWriter* writer, const SerdNode* uri)
 }
 
 SerdStatus
-serd_writer_set_prefix(SerdWriter*     writer,
-                       const SerdNode* name,
-                       const SerdNode* uri)
+serd_writer_set_prefix(SerdWriter* const     writer,
+                       const SerdNode* const name,
+                       const SerdNode* const uri)
 {
+  assert(writer);
+  assert(name);
+  assert(uri);
+
   SerdStatus st = SERD_SUCCESS;
 
   TRY(st, serd_env_set_prefix(writer->env, name, uri));
@@ -1246,7 +1231,6 @@ serd_writer_set_prefix(SerdWriter*     writer,
     TRY(st, esink(": <", 3, writer));
     TRY(st, ewrite_uri(writer, uri->buf, uri->n_bytes));
     TRY(st, esink(">", 1, writer));
-    writer->last_sep = SEP_NODE;
     TRY(st, write_sep(writer, SEP_END_DIRECT));
   }
 
@@ -1254,7 +1238,7 @@ serd_writer_set_prefix(SerdWriter*     writer,
 }
 
 void
-serd_writer_free(SerdWriter* writer)
+serd_writer_free(SerdWriter* const writer)
 {
   if (!writer) {
     return;
@@ -1273,20 +1257,26 @@ serd_writer_free(SerdWriter* writer)
 }
 
 SerdEnv*
-serd_writer_get_env(SerdWriter* writer)
+serd_writer_get_env(SerdWriter* const writer)
 {
+  assert(writer);
   return writer->env;
 }
 
 size_t
-serd_file_sink(const void* buf, size_t len, void* stream)
+serd_file_sink(const void* const buf, const size_t len, void* const stream)
 {
+  assert(buf);
+  assert(stream);
   return fwrite(buf, 1, len, (FILE*)stream);
 }
 
 size_t
-serd_chunk_sink(const void* buf, size_t len, void* stream)
+serd_chunk_sink(const void* const buf, const size_t len, void* const stream)
 {
+  assert(buf);
+  assert(stream);
+
   SerdChunk* chunk = (SerdChunk*)stream;
   uint8_t* new_buf = (uint8_t*)realloc((uint8_t*)chunk->buf, chunk->len + len);
   if (new_buf) {
@@ -1298,8 +1288,9 @@ serd_chunk_sink(const void* buf, size_t len, void* stream)
 }
 
 uint8_t*
-serd_chunk_sink_finish(SerdChunk* stream)
+serd_chunk_sink_finish(SerdChunk* const stream)
 {
+  assert(stream);
   serd_chunk_sink("", 1, stream);
   return (uint8_t*)stream->buf;
 }
