@@ -3,20 +3,14 @@
 # Copyright 2022 David Robillard <d@drobilla.net>
 # SPDX-License-Identifier: ISC
 
-"""Test serdi quiet option."""
+"""Test quiet command-line option."""
 
-import argparse
-import sys
 import shlex
 import subprocess
 
-parser = argparse.ArgumentParser(description=__doc__)
+import serd_test_util as util
 
-parser.add_argument("--serdi", default="./serdi", help="path to serdi")
-parser.add_argument("--wrapper", default="", help="executable wrapper")
-parser.add_argument("input", help="invalid input file")
-
-args = parser.parse_args(sys.argv[1:])
+args = util.wrapper_args(__doc__, True)
 command = shlex.split(args.wrapper) + [args.serdi, "-q", args.input]
 proc = subprocess.run(
     command, check=False, stdout=subprocess.PIPE, stderr=subprocess.PIPE
