@@ -3,13 +3,15 @@
 
 #undef NDEBUG
 
-#include "serd/serd.h"
+#include <serd/serd.h>
 
 #include <assert.h>
 #include <stdint.h>
 #include <stdio.h>
 #include <string.h>
 
+#define NS_EG "http://example.org/"
+
 #define USTR(s) ((const uint8_t*)(s))
 
 static void
@@ -22,8 +24,8 @@ test_write_long_literal(void)
 
   assert(writer);
 
-  SerdNode s = serd_node_from_string(SERD_URI, USTR("http://example.org/s"));
-  SerdNode p = serd_node_from_string(SERD_URI, USTR("http://example.org/p"));
+  SerdNode s = serd_node_from_string(SERD_URI, USTR(NS_EG "s"));
+  SerdNode p = serd_node_from_string(SERD_URI, USTR(NS_EG "p"));
   SerdNode o =
     serd_node_from_string(SERD_LITERAL, USTR("hello \"\"\"world\"\"\"!"));
 
@@ -52,16 +54,16 @@ test_write_nested_anon(void)
 
   assert(writer);
 
-  SerdNode s0  = serd_node_from_string(SERD_URI, USTR("http://example.org/s0"));
-  SerdNode p0  = serd_node_from_string(SERD_URI, USTR("http://example.org/p0"));
+  SerdNode s0  = serd_node_from_string(SERD_URI, USTR(NS_EG "s0"));
+  SerdNode p0  = serd_node_from_string(SERD_URI, USTR(NS_EG "p0"));
   SerdNode b0  = serd_node_from_string(SERD_BLANK, USTR("b0"));
-  SerdNode p1  = serd_node_from_string(SERD_URI, USTR("http://example.org/p1"));
+  SerdNode p1  = serd_node_from_string(SERD_URI, USTR(NS_EG "p1"));
   SerdNode b1  = serd_node_from_string(SERD_BLANK, USTR("b1"));
-  SerdNode p2  = serd_node_from_string(SERD_URI, USTR("http://example.org/p2"));
-  SerdNode o2  = serd_node_from_string(SERD_URI, USTR("http://example.org/o2"));
-  SerdNode p3  = serd_node_from_string(SERD_URI, USTR("http://example.org/p3"));
-  SerdNode p4  = serd_node_from_string(SERD_URI, USTR("http://example.org/p4"));
-  SerdNode o4  = serd_node_from_string(SERD_URI, USTR("http://example.org/o4"));
+  SerdNode p2  = serd_node_from_string(SERD_URI, USTR(NS_EG "p2"));
+  SerdNode o2  = serd_node_from_string(SERD_URI, USTR(NS_EG "o2"));
+  SerdNode p3  = serd_node_from_string(SERD_URI, USTR(NS_EG "p3"));
+  SerdNode p4  = serd_node_from_string(SERD_URI, USTR(NS_EG "p4"));
+  SerdNode o4  = serd_node_from_string(SERD_URI, USTR(NS_EG "o4"));
   SerdNode nil = serd_node_from_string(
     SERD_URI, USTR("http://www.w3.org/1999/02/22-rdf-syntax-ns#nil"));
 
@@ -110,7 +112,6 @@ test_write_nested_anon(void)
     "\t\t<http://example.org/p4> <http://example.org/o4>\n"
     "\t] .\n";
 
-  fprintf(stderr, "%s\n", out);
   assert(!strcmp((char*)out, expected));
   serd_free(out);
 }
@@ -132,9 +133,11 @@ test_writer_cleanup(void)
   SerdWriter* writer =
     serd_writer_new(SERD_TURTLE, (SerdStyle)0U, env, NULL, null_sink, NULL);
 
-  SerdNode s = serd_node_from_string(SERD_URI, USTR("http://example.org/s"));
-  SerdNode p = serd_node_from_string(SERD_URI, USTR("http://example.org/p"));
-  SerdNode o = serd_node_from_string(SERD_BLANK, USTR("http://example.org/o"));
+  SerdNode s = serd_node_from_string(SERD_URI, USTR(NS_EG "s"));
+  SerdNode p = serd_node_from_string(SERD_URI, USTR(NS_EG "p"));
+
+  char     o_buf[12] = {'b', '0', '\0'};
+  SerdNode o         = serd_node_from_string(SERD_BLANK, USTR(o_buf));
 
   st = serd_writer_write_statement(
     writer, SERD_ANON_O_BEGIN, NULL, &s, &p, &o, NULL, NULL);
@@ -142,29 +145,70 @@ test_writer_cleanup(void)
   assert(!st);
 
   // Write the start of several nested anonymous objects
-  for (unsigned i = 0U; !st && i < 8U; ++i) {
-    char buf[12] = {0};
-    snprintf(buf, sizeof(buf), "b%u", i);
+  for (unsigned i = 1U; !st && i < 9U; ++i) {
+    char next_o_buf[12] = {'\0'};
+    snprintf(next_o_buf, sizeof(next_o_buf), "b%u", i);
 
-    SerdNode next_o = serd_node_from_string(SERD_BLANK, USTR(buf));
+    SerdNode next_o = serd_node_from_string(SERD_BLANK, USTR(next_o_buf));
 
-    st = serd_writer_write_statement(
-      writer, SERD_ANON_O_BEGIN, NULL, &o, &p, &next_o, NULL, NULL);
+    st = serd_writer_write_statement(writer,
+                                     SERD_ANON_O_BEGIN | SERD_ANON_CONT,
+                                     NULL,
+                                     &o,
+                                     &p,
+                                     &next_o,
+                                     NULL,
+                                     NULL);
 
-    o = next_o;
+    assert(!st);
+
+    memcpy(o_buf, next_o_buf, sizeof(o_buf));
   }
 
   // Finish writing without terminating nodes
-  assert(!(st = serd_writer_finish(writer)));
+  st = serd_writer_finish(writer);
+  assert(!st);
 
   // Set the base to an empty URI
-  assert(!(st = serd_writer_set_base_uri(writer, NULL)));
+  st = serd_writer_set_base_uri(writer, NULL);
+  assert(!st);
 
   // Free (which could leak if the writer doesn't clean up the stack properly)
   serd_writer_free(writer);
   serd_env_free(env);
 }
 
+static void
+test_write_bad_anon_stack(void)
+{
+  SerdStatus  st  = SERD_SUCCESS;
+  SerdEnv*    env = serd_env_new(NULL);
+  SerdWriter* writer =
+    serd_writer_new(SERD_TURTLE, (SerdStyle)0U, env, NULL, null_sink, NULL);
+
+  SerdNode s  = serd_node_from_string(SERD_URI, USTR(NS_EG "s"));
+  SerdNode p  = serd_node_from_string(SERD_URI, USTR(NS_EG "p"));
+  SerdNode b0 = serd_node_from_string(SERD_BLANK, USTR("b0"));
+  SerdNode b1 = serd_node_from_string(SERD_BLANK, USTR("b1"));
+  SerdNode b2 = serd_node_from_string(SERD_BLANK, USTR("b2"));
+
+  st = serd_writer_write_statement(
+    writer, SERD_ANON_O_BEGIN, NULL, &s, &p, &b0, NULL, NULL);
+  assert(!st);
+
+  // (missing call to end the anonymous node here)
+
+  st = serd_writer_write_statement(
+    writer, SERD_ANON_O_BEGIN, NULL, &b1, &p, &b2, NULL, NULL);
+
+  assert(st == SERD_ERR_BAD_ARG);
+
+  st = serd_writer_finish(writer);
+  assert(!st);
+  serd_writer_free(writer);
+  serd_env_free(env);
+}
+
 static void
 test_strict_write(void)
 {
@@ -180,8 +224,8 @@ test_strict_write(void)
 
   const uint8_t bad_str[] = {0xFF, 0x90, 'h', 'i', 0};
 
-  SerdNode s = serd_node_from_string(SERD_URI, USTR("http://example.org/s"));
-  SerdNode p = serd_node_from_string(SERD_URI, USTR("http://example.org/p"));
+  SerdNode s = serd_node_from_string(SERD_URI, USTR(NS_EG "s"));
+  SerdNode p = serd_node_from_string(SERD_URI, USTR(NS_EG "p"));
 
   SerdNode bad_lit = serd_node_from_string(SERD_LITERAL, bad_str);
   SerdNode bad_uri = serd_node_from_string(SERD_URI, bad_str);
@@ -194,8 +238,9 @@ test_strict_write(void)
 
   serd_writer_free(writer);
   serd_env_free(env);
-  fclose(fd);
-  remove(path);
+
+  assert(!fclose(fd));
+  assert(!remove(path));
 }
 
 // Produce a write error without setting errno
@@ -233,8 +278,11 @@ main(void)
   test_write_long_literal();
   test_write_nested_anon();
   test_writer_cleanup();
+  test_write_bad_anon_stack();
   test_strict_write();
   test_write_error();
 
   return 0;
 }
+
+#undef NS_EG
